@@ -207,6 +207,10 @@ pub struct StockPiece {
 
     /// Pattern direction of stock piece.
     pub pattern_direction: PatternDirection,
+
+    /// Price to use to optimize for price when not all stock pieces are the same price per unit
+    /// area. If optimizing for less waste instead, price can be set to 0 for all stock pieces.
+    pub price: usize,
 }
 
 impl StockPiece {
@@ -328,11 +332,14 @@ trait Bin {
         length: usize,
         blade_width: usize,
         pattern_direction: PatternDirection,
+        price: usize,
     ) -> Self;
 
     /// Computes the fitness of this `Bin` on a scale of 0.0 to 1.0, with 1.0 being the most fit.
     fn fitness(&self) -> f64;
 
+    fn price(&self) -> usize;
+
     /// Removes `UsedCutPiece`s from this `Bin` and returns how many were removed.
     fn remove_cut_pieces<I>(&mut self, cut_pieces: I) -> usize
     where
@@ -546,6 +553,7 @@ where
                 stock_piece.length,
                 self.blade_width,
                 stock_piece.pattern_direction,
+                stock_piece.price,
             );
             if !bin.insert_cut_piece_random_heuristic(cut_piece, rng) {
                 return false;
@@ -673,6 +681,9 @@ pub struct Solution {
 
     /// The stock pieces that were used for this solution, each containing the demand piece layout.
     pub stock_pieces: Vec<ResultStockPiece>,
+
+    #[cfg_attr(feature = "serialize", serde(skip))]
+    price: usize,
 }
 
 /// Optimizer for optimizing rectangular cut pieces from rectangular
@@ -759,8 +770,8 @@ impl Optimizer {
         self.random_seed = seed;
         self
     }
-    
-    /// Set whether the optimizer should allow mixed sized stock pieces in the results. 
+
+    /// Set whether the optimizer should allow mixed sized stock pieces in the results.
     /// If set to false, and multiple stock sizes are given, only one stock size will be used in
     /// the results.
     pub fn allow_mixed_stock_sizes(&mut self, allow: bool) -> &mut Self {
@@ -776,11 +787,7 @@ impl Optimizer {
     where
         F: Fn(f64),
     {
-        let (fitness, stock_pieces) = self.optimize::<GuillotineBin, F>(progress_callback)?;
-        Ok(Solution {
-            fitness,
-            stock_pieces,
-        })
+        self.optimize::<GuillotineBin, F>(progress_callback)
     }
 
     /// Optimize without the requirement of guillotine cuts. Cuts can start and stop in the middle
@@ -791,21 +798,21 @@ impl Optimizer {
     where
         F: Fn(f64),
     {
-        let (fitness, stock_pieces) = self.optimize::<MaxRectsBin, F>(progress_callback)?;
-        Ok(Solution {
-            fitness,
-            stock_pieces,
-        })
+        self.optimize::<MaxRectsBin, F>(progress_callback)
     }
 
-    fn optimize<B, F>(&self, progress_callback: F) -> Result<(f64, Vec<ResultStockPiece>)>
+    fn optimize<B, F>(&self, progress_callback: F) -> Result<Solution>
     where
         B: Bin + Clone + Send + Into<ResultStockPiece>,
         F: Fn(f64),
     {
         // If there are no cut pieces, there's nothing to optimize.
         if self.cut_pieces.is_empty() {
-            return Ok((1.0, vec![]))
+            return Ok(Solution {
+                fitness: 1.0,
+                stock_pieces: Vec::new(),
+                price: 0,
+            });
         }
 
         let size_set: FnvHashSet<(usize, usize)> = self
@@ -844,24 +851,31 @@ impl Optimizer {
                 .collect();
 
             let completed_runs = i + 1;
-            if let Ok((fitness, used_stock_pieces)) =
+            if let Ok(solution) =
                 self.optimize_with_stock_pieces::<B, _>(&stock_pieces, &|progress| {
                     progress_callback((completed_runs as f64 + progress) / num_runs as f64);
                 })
             {
                 match best_result {
-                    Ok((best_fitness, _)) => {
-                        if fitness > best_fitness {
-                            best_result = Ok((fitness, used_stock_pieces));
+                    Ok(ref best_solution) => {
+                        // Use the lower-priced solution, but if the prices are the same, use the
+                        // solution with the higher fitness score.
+                        if solution.price < best_solution.price
+                            || (solution.price == best_solution.price
+                                && solution.fitness > best_solution.fitness)
+                        {
+                            best_result = Ok(solution);
                         }
                     }
-                    Err(_) => best_result = Ok((fitness, used_stock_pieces)),
+                    Err(_) => best_result = Ok(solution),
                 }
             }
         }
 
-        if let Ok((_, ref mut used_stock_pieces)) = &mut best_result {
-            used_stock_pieces.sort_by_key(|p| cmp::Reverse((p.width, p.length)));
+        if let Ok(ref mut solution) = &mut best_result {
+            solution
+                .stock_pieces
+                .sort_by_key(|p| cmp::Reverse((p.width, p.length)));
         };
 
         best_result
@@ -871,7 +885,7 @@ impl Optimizer {
         &self,
         stock_pieces: &[StockPiece],
         progress_callback: &F,
-    ) -> Result<(f64, Vec<ResultStockPiece>)>
+    ) -> Result<Solution>
     where
         B: Bin + Clone + Send + Into<ResultStockPiece>,
         F: Fn(f64),
@@ -897,10 +911,16 @@ impl Optimizer {
         let best_unit = &mut result_units[0];
         let fitness = best_unit.fitness();
 
+        let price = best_unit.bins.iter().map(|bin| bin.price()).sum();
+
         let used_stock_pieces: Vec<ResultStockPiece> =
             best_unit.bins.drain(..).map(Into::into).collect();
 
-        Ok((fitness, used_stock_pieces))
+        Ok(Solution {
+            fitness,
+            stock_pieces: used_stock_pieces,
+            price,
+        })
     }
 }
 
@@ -913,11 +933,13 @@ mod test {
             width: 48,
             length: 96,
             pattern_direction: PatternDirection::None,
+            price: 0,
         },
         StockPiece {
             width: 48,
             length: 120,
             pattern_direction: PatternDirection::None,
+            price: 0,
         },
     ];
 
@@ -971,6 +993,7 @@ mod test {
                 width: 10,
                 length: 11,
                 pattern_direction: PatternDirection::None,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -993,6 +1016,7 @@ mod test {
                 width: 10,
                 length: 11,
                 pattern_direction: PatternDirection::ParallelToWidth,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1015,6 +1039,7 @@ mod test {
                 width: 10,
                 length: 10,
                 pattern_direction: PatternDirection::None,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1042,6 +1067,7 @@ mod test {
                 width: 10,
                 length: 11,
                 pattern_direction: PatternDirection::None,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1069,6 +1095,7 @@ mod test {
                 width: 10,
                 length: 11,
                 pattern_direction: PatternDirection::ParallelToWidth,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1096,6 +1123,7 @@ mod test {
                 width: 100,
                 length: 100,
                 pattern_direction: PatternDirection::None,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1135,6 +1163,7 @@ mod test {
                 width: 10,
                 length: 11,
                 pattern_direction: PatternDirection::None,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1157,6 +1186,7 @@ mod test {
                 width: 10,
                 length: 11,
                 pattern_direction: PatternDirection::ParallelToWidth,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1179,6 +1209,7 @@ mod test {
                 width: 10,
                 length: 10,
                 pattern_direction: PatternDirection::None,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1206,6 +1237,7 @@ mod test {
                 width: 10,
                 length: 11,
                 pattern_direction: PatternDirection::None,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1233,6 +1265,7 @@ mod test {
                 width: 10,
                 length: 11,
                 pattern_direction: PatternDirection::ParallelToWidth,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1260,6 +1293,7 @@ mod test {
                 width: 100,
                 length: 100,
                 pattern_direction: PatternDirection::None,
+                price: 0,
             })
             .add_cut_piece(CutPiece {
                 external_id: Some(1),
@@ -1312,4 +1346,91 @@ mod test {
             }
         }
     }
+
+    #[test]
+    fn test_different_stock_piece_prices() {
+        let result = Optimizer::new()
+            .add_stock_piece(StockPiece{
+                width: 48,
+                length: 96,
+                pattern_direction: PatternDirection::None,
+                price: 1,
+            })
+            .add_stock_piece(StockPiece{
+                width: 48,
+                length: 120,
+                pattern_direction: PatternDirection::None,
+                // Maker the 48x120 stock piece more expensive than (2) 48x96 pieces.
+                price: 3,
+            })
+            .add_cut_piece(CutPiece {
+                external_id: Some(1),
+                width: 48,
+                length: 50,
+                pattern_direction: PatternDirection::None,
+                can_rotate: false,
+            })
+            .add_cut_piece(CutPiece {
+                external_id: Some(2),
+                width: 48,
+                length: 50,
+                pattern_direction: PatternDirection::None,
+                can_rotate: false,
+            })
+            .set_cut_width(1)
+            .set_random_seed(1)
+            .allow_mixed_stock_sizes(false)
+            .optimize_guillotine(|_| {});
+
+        assert!(result.is_ok());
+        if let Ok(solution) = result {
+            // A single 48x120 stock piece could be used, but since we've set (2) 48x96 pieces to
+            // be a lower price than (1) 48x120, it should use (2) 48x96 pieces instead.
+            assert_eq!(solution.stock_pieces.len(), 2);
+            for stock_piece in solution.stock_pieces {
+                assert_eq!(stock_piece.length, 96)
+            }
+        }
+    }
+
+    #[test]
+    fn test_same_stock_piece_prices() {
+        let result = Optimizer::new()
+            .add_stock_piece(StockPiece{
+                width: 48,
+                length: 96,
+                pattern_direction: PatternDirection::None,
+                price: 0,
+            })
+            .add_stock_piece(StockPiece{
+                width: 48,
+                length: 120,
+                pattern_direction: PatternDirection::None,
+                price: 0,
+            })
+            .add_cut_piece(CutPiece {
+                external_id: Some(1),
+                width: 48,
+                length: 50,
+                pattern_direction: PatternDirection::None,
+                can_rotate: false,
+            })
+            .add_cut_piece(CutPiece {
+                external_id: Some(2),
+                width: 48,
+                length: 50,
+                pattern_direction: PatternDirection::None,
+                can_rotate: false,
+            })
+            .set_cut_width(1)
+            .set_random_seed(1)
+            .allow_mixed_stock_sizes(false)
+            .optimize_guillotine(|_| {});
+
+        assert!(result.is_ok());
+        if let Ok(solution) = result {
+            assert_eq!(solution.stock_pieces.len(), 1);
+            assert_eq!(solution.stock_pieces[0].length, 120)
+        }
+    }
 }
